@@ -1,10 +1,15 @@
 #												vim:sw=4:ts=4
-# $Id: basic.rb,v 1.9 2003/10/17 01:34:34 whythluckystiff Exp $
+# $Id$
 #
 require 'runit/testcase'
 require 'runit/cui/testrunner'
 require 'yaml'
 
+# [ruby-core:01946]
+module YAML_Tests
+    StructTest = Struct::new( :c )
+end
+
 class YAML_Unit_Tests < RUNIT::TestCase
 	#
 	# Convert between YAML and the object to verify correct parsing and
@@ -12,20 +17,26 @@ class YAML_Unit_Tests < RUNIT::TestCase
 	#
 	def assert_to_yaml( obj, yaml )
 		assert_equal( obj, YAML::load( yaml ) )
-		assert_equal( obj, YAML::parse( yaml ).transform )
+        assert_equal( obj, YAML::parse( yaml ).transform )
         assert_equal( obj, YAML::load( obj.to_yaml ) )
-		assert_equal( obj, YAML::parse( obj.to_yaml ).transform )
+        assert_equal( obj, YAML::parse( obj.to_yaml ).transform )
         assert_equal( obj, YAML::load(
 			obj.to_yaml( :UseVersion => true, :UseHeader => true, :SortKeys => true ) 
 		) )
-        assert_equal( obj, YAML::Syck::Parser.new( :Input => :Bytecode ).load( YAML::Syck::compile( yaml ) ) )
+
+        # yb = YAML::Parser.new
+        # yb.resolver = YAML.resolver
+        # yb.input = :bytecode
+        # assert_equal( obj, yb.load( YAML::Syck::compile( yaml ) ) )
 	end
 
     #
     # Test bytecode parser
     #
     def assert_bytecode( obj, yaml )
-		assert_equal( obj, YAML::Syck::Parser.new( :Input => :Bytecode ).load( yaml ) )
+        yp = YAML::Parser.new
+        yp.input = :bytecode
+        assert_equal( obj, yp.load( yaml ) )
     end
 
 	#
@@ -33,8 +44,10 @@ class YAML_Unit_Tests < RUNIT::TestCase
 	#
 	def assert_parse_only( obj, yaml )
 		assert_equal( obj, YAML::load( yaml ) )
-		assert_equal( obj, YAML::parse( yaml ).transform )
-        # assert_equal( obj, YAML::Syck::Parser.new( :Input => :Bytecode ).load( YAML::Syck::compile( yaml ) ) )
+        assert_equal( obj, YAML::parse( yaml ).transform )
+        yp = YAML::Parser.new
+        yp.input = :bytecode
+        assert_equal( obj, yp.load( YAML::Syck::compile( yaml ) ) )
 	end
 
     def assert_path_segments( path, segments )
@@ -65,7 +78,7 @@ class YAML_Unit_Tests < RUNIT::TestCase
 	def test_basic_map
 		# Simple map
         map = { 'one' => 'foo', 'three' => 'baz', 'two' => 'bar' }
-		assert_parse_only(
+		assert_to_yaml(
 			map, <<EOY
 one: foo
 two: bar
@@ -80,7 +93,7 @@ EOY
 		basic = { 1 => 'simple string', 2 => 42, 3 => '1 Single Quoted String',
 			  4 => 'YAML\'s Double "Quoted" String', 5 => "A block\n  with several\n    lines.\n",
 			  6 => "A \"chomped\" block", 7 => "A folded\n string\n" }
-		assert_parse_only(
+		assert_to_yaml(
 			basic, <<EOY
 1: simple string
 2: 42
@@ -186,7 +199,7 @@ EOY
 			[ 'Mark McGwire', 65, 0.278 ],
 			[ 'Sammy Sosa', 63, 0.288 ]
 		]
-		assert_parse_only(
+		assert_to_yaml(
 		  seq, <<EOY
 - [ name         , hr , avg   ]
 - [ Mark McGwire , 65 , 0.278 ]
@@ -204,7 +217,7 @@ EOY
 			'Sammy Sosa' =>
 		    { 'hr' => 63, 'avg' => 0.288 }
 		}
-		assert_parse_only(
+		assert_to_yaml(
 		  map, <<EOY
 Mark McGwire: {hr: 65, avg: 0.278}
 Sammy Sosa:   {hr: 63,
@@ -215,11 +228,17 @@ EOY
             "SSammy Sosa\nM\nShr\nS63\nSavg\nS0.288\nE\nE\n" )
 	end
 
+    def test_ambiguous_comments
+        assert_to_yaml( "Call the method #dave", <<EOY )
+--- "Call the method #dave"
+EOY
+    end
+
 	def test_spec_nested_comments
 		# Map and sequences with comments
         nest = { 'hr' => [ 'Mark McGwire', 'Sammy Sosa' ],
 		    'rbi' => [ 'Sammy Sosa', 'Ken Griffey' ] }
-		assert_parse_only(
+		assert_to_yaml(
 		  nest, <<EOY
 hr: # 1998 hr ranking
   - Mark McGwire
@@ -240,7 +259,7 @@ EOY
 			  [ 'Mark McGwire', 'Sammy Sosa' ],
 			  'rbi' =>
 			  [ 'Sammy Sosa', 'Ken Griffey' ] }
-		assert_parse_only( anc1, <<EOY )
+		assert_to_yaml( anc1, <<EOY )
 hr:
    - Mark McGwire
    # Name "Sammy Sosa" scalar SS
@@ -308,7 +327,7 @@ EOY
 		dj = Date.new( 2001, 7, 23 )
         complex = { [ 'Detroit Tigers', 'Chicago Cubs' ] => [ Date.new( 2001, 7, 23 ) ],
 			  [ 'New York Yankees', 'Atlanta Braves' ] => [ Date.new( 2001, 7, 2 ), Date.new( 2001, 8, 12 ), Date.new( 2001, 8, 14 ) ] }
-		assert_parse_only(
+		assert_to_yaml(
 			complex, <<EOY
 ? # PLAY SCHEDULE
   - Detroit Tigers
@@ -324,7 +343,7 @@ EOY
 		)
 
 		# Complex key #2
-		assert_parse_only(
+		assert_to_yaml(
 		  { [ 'New York Yankees', 'Atlanta Braves' ] =>
 		    [ Date.new( 2001, 7, 2 ), Date.new( 2001, 8, 12 ),
 			  Date.new( 2001, 8, 14 ) ],
@@ -354,7 +373,7 @@ EOY
 			[ { 'item' => 'Super Hoop', 'quantity' => 1 },
 			  { 'item' => 'Basketball', 'quantity' => 4 },
 			  { 'item' => 'Big Shoes', 'quantity' => 1 } ] }
-		assert_parse_only(
+		assert_to_yaml(
 		  seq, <<EOY
 invoice: 34843
 date   : 2001-01-23
@@ -377,7 +396,7 @@ EOY
     def test_spec_sequence_in_sequence_shortcut
         # Seq-in-seq
         seq = [ [ [ 'one', 'two', 'three' ] ] ]
-        assert_parse_only( seq, <<EOY )
+        assert_to_yaml( seq, <<EOY )
 - - - one
     - two
     - three
@@ -398,7 +417,7 @@ EOY
           ],
           [ 'eight', 'nine' ]
         ]
-        assert_parse_only( seq, <<EOY )
+        assert_to_yaml( seq, <<EOY )
 - - - - one
   - - two
     - three
@@ -418,7 +437,7 @@ EOY
 	def test_spec_single_literal
 		# Literal scalar block
         lit = [ "\\/|\\/|\n/ |  |_\n" ]
-		assert_parse_only( lit, <<EOY )
+		assert_to_yaml( lit, <<EOY )
 - |
     \\/|\\/|
     / |  |_
@@ -429,14 +448,308 @@ EOY
 	def test_spec_single_folded
 		# Folded scalar block
         fold = [ "Mark McGwire's year was crippled by a knee injury.\n" ]
-		assert_parse_only(
-			fold, <<EOY
+		assert_to_yaml( fold, <<EOY )
 - >
     Mark McGwire's
     year was crippled
     by a knee injury.
 EOY
-		)
+
+        # Force a few elaborate folded blocks
+		assert_to_yaml( [ <<STR1, <<STR2, <<STR3 ], <<EOY )
+Here's what you're going to need:
+
+
+* Ruby 1.8.0 or higher.
+
+** Linux/FreeBSD: "Get the latest, please.":http://ruby-lang.org/en/20020102.html
+
+** Windows: "Installer for Windows.":http://rubyinstaller.sourceforge.net/
+
+** Mac: "OSX disk image.":http://homepage.mac.com/discord/Ruby/
+
+* Once Ruby is installed, open a command prompt and type:
+
+
+<pre>
+  ruby -ropen-uri -e 'eval(open("http://go.hobix.com/").read)'
+</pre>
+
+STR1
+Ok, so the idea here is that one whole weblahhg is contained
+in a single directory.  What is stored in the directory?
+
+
+<pre>
+  hobix.yaml <- configuration
+
+  entries/   <- edit and organize
+                your news items,
+                articles and so on.
+
+  skel/      <- contains your
+                templates
+
+  htdocs/    <- html is created here,
+                store all your images here,
+                this is your viewable
+                websyht
+
+  lib/       <- extra hobix libraries
+                (plugins) go here
+</pre>
+
+
+One weblahhg can be shared with many authors.  In the @hobix.yaml@
+file, you can store information about each author, as well as 
+information about others who contribute to your websyht.
+
+
+You also have a file of your own, a configuration file called
+@.hobixrc@, which contains a list of the weblahhgs you belong
+to.
+
+
+h2. Pull One From the Sky
+
+
+If you would like to create your own weblahhg from scratch:
+
+
+<pre>
+  hobix create blahhg
+</pre>
+
+
+You will be prompted for a full path where the new weblahhg can
+be created.  Don't worry if the directory doesn't yet exist,
+Hobix will take care of clearing a path for it.
+
+
+Once you give it the path, Hobix will create all the necessary
+directories, as well as the @hobix.yaml@.  You should also have
+a set of sample templates to get you started.
+
+
+In fact, if you want to generate the default site:
+
+
+<pre>
+  hobix regen blahhg
+</pre>
+
+
+h2. Joining Hands With Others
+
+
+To join an existing weblahhg:
+
+
+<pre>
+  hobix add other-blahhg /path/to/other-blahhg/hobix.yaml
+</pre>
+
+
+You don't need to be on a weblahhg's author list to join the weblahhg.
+You just need permissions to edit the file.
+
+
+h2. Leaving in a Cloud of Keystrokes
+
+
+To remove a weblahhg from your configuration:
+
+
+<pre>
+  hobix del other-blahhg
+</pre>
+
+
+Please don't be afraid to edit your configuration file yourself,
+should the commandline not suit your style.
+
+
+See, here's my @.hobixrc@:
+
+
+<pre>
+  --- 
+  weblogs: 
+    hobix: /usr/local/www/hobix.com/www/hobix.yaml
+    why: /usr/local/www/whytheluckystiff.net/www/hobix.yaml
+  username: why
+  use editor: true
+</pre>
+
+
+That's a YAML file.  Very simple to edit.  You can manually edit
+your information, safely add or edit your weblogs, and save back
+to your @.hobixrc@.
+
+
+Enough then.  Time to trick out your new Hoblahhg.
+You will be guided through an automatic installation (or upgrade) of
+Hobix.
+STR2
+NOW BUNKING together: the ruby and yaml scripts up at http://go.hobix.com/. Hey ,, the life and unity we feel is tree-mending-US!!
+
+Try:
+
+<pre>
+  ruby -ropen-uri -e'eval open("http://go.hobix.com/").read'
+</pre>
+
+No longer are curtains dropping ON the actors' arms !!  No longer are swift currents ON the actors' legs !!  The actors have a bottomless cereal container, witness.
+STR3
+--- 
+- |+
+  Here's what you're going to need:
+
+
+  * Ruby 1.8.0 or higher.
+
+  ** Linux/FreeBSD: "Get the latest, please.":http://ruby-lang.org/en/20020102.html
+
+  ** Windows: "Installer for Windows.":http://rubyinstaller.sourceforge.net/
+
+  ** Mac: "OSX disk image.":http://homepage.mac.com/discord/Ruby/
+
+  * Once Ruby is installed, open a command prompt and type:
+
+
+  <pre>
+    ruby -ropen-uri -e 'eval(open("http://go.hobix.com/").read)'
+  </pre>
+
+- |
+  Ok, so the idea here is that one whole weblahhg is contained
+  in a single directory.  What is stored in the directory?
+
+
+  <pre>
+    hobix.yaml <- configuration
+
+    entries/   <- edit and organize
+                  your news items,
+                  articles and so on.
+
+    skel/      <- contains your
+                  templates
+
+    htdocs/    <- html is created here,
+                  store all your images here,
+                  this is your viewable
+                  websyht
+
+    lib/       <- extra hobix libraries
+                  (plugins) go here
+  </pre>
+
+
+  One weblahhg can be shared with many authors.  In the @hobix.yaml@
+  file, you can store information about each author, as well as 
+  information about others who contribute to your websyht.
+
+
+  You also have a file of your own, a configuration file called
+  @.hobixrc@, which contains a list of the weblahhgs you belong
+  to.
+
+
+  h2. Pull One From the Sky
+
+
+  If you would like to create your own weblahhg from scratch:
+
+
+  <pre>
+    hobix create blahhg
+  </pre>
+
+
+  You will be prompted for a full path where the new weblahhg can
+  be created.  Don't worry if the directory doesn't yet exist,
+  Hobix will take care of clearing a path for it.
+
+
+  Once you give it the path, Hobix will create all the necessary
+  directories, as well as the @hobix.yaml@.  You should also have
+  a set of sample templates to get you started.
+
+
+  In fact, if you want to generate the default site:
+
+
+  <pre>
+    hobix regen blahhg
+  </pre>
+
+
+  h2. Joining Hands With Others
+
+
+  To join an existing weblahhg:
+
+
+  <pre>
+    hobix add other-blahhg /path/to/other-blahhg/hobix.yaml
+  </pre>
+
+
+  You don't need to be on a weblahhg's author list to join the weblahhg.
+  You just need permissions to edit the file.
+
+
+  h2. Leaving in a Cloud of Keystrokes
+
+
+  To remove a weblahhg from your configuration:
+
+
+  <pre>
+    hobix del other-blahhg
+  </pre>
+
+
+  Please don't be afraid to edit your configuration file yourself,
+  should the commandline not suit your style.
+
+
+  See, here's my @.hobixrc@:
+
+
+  <pre>
+    --- 
+    weblogs: 
+      hobix: /usr/local/www/hobix.com/www/hobix.yaml
+      why: /usr/local/www/whytheluckystiff.net/www/hobix.yaml
+    username: why
+    use editor: true
+  </pre>
+
+
+  That's a YAML file.  Very simple to edit.  You can manually edit
+  your information, safely add or edit your weblogs, and save back
+  to your @.hobixrc@.
+
+
+  Enough then.  Time to trick out your new Hoblahhg.
+  You will be guided through an automatic installation (or upgrade) of
+  Hobix.
+
+
+- |
+  NOW BUNKING together: the ruby and yaml scripts up at http://go.hobix.com/. Hey ,, the life and unity we feel is tree-mending-US!!
+
+  Try:
+
+  <pre>
+    ruby -ropen-uri -e'eval open("http://go.hobix.com/").read'
+  </pre>
+
+  No longer are curtains dropping ON the actors' arms !!  No longer are swift currents ON the actors' legs !!  The actors have a bottomless cereal container, witness.
+
+EOY
 
         assert_bytecode( fold, "D\nQ\nSMark McGwire's year was crippled by a knee injury.\nN\nE\n" )
 	end
@@ -444,7 +757,7 @@ EOY
 	def test_spec_preserve_indent
 		# Preserve indented spaces
         fold = "Sammy Sosa completed another fine season with great stats.\n\n  63 Home Runs\n  0.288 Batting Average\n\nWhat a year!\n"
-		assert_parse_only( fold, <<EOY )
+		assert_to_yaml( fold, <<EOY )
 --- >
  Sammy Sosa completed another
  fine season with great stats.
@@ -461,7 +774,7 @@ EOY
 	def test_spec_indentation_determines_scope
         map = { 'name' => 'Mark McGwire', 'accomplishment' => "Mark set a major league home run record in 1998.\n",
 			  'stats' => "65 Home Runs\n0.278 Batting Average\n" }
-		assert_parse_only( map, <<EOY )
+		assert_to_yaml( map, <<EOY )
 name: Mark McGwire
 accomplishment: >
    Mark set a major league
@@ -473,11 +786,25 @@ EOY
         assert_bytecode( map, "D\nM\nSname\nSMark McGwire\nSaccomplishment\nSMark set a major league home run record in 1998.\nN\nSstats\nS65 Home Runs\nN\nC0.278 Batting Average\nN\nE\n" )
 	end
 
+    #	def test_spec_quoted_scalars
+    #		assert_to_yaml(
+    #			{"tie-fighter"=>"|\\-*-/|", "control"=>"\0101998\t1999\t2000\n", "unicode"=>"Sosa did fine." + ["263A".hex].pack('U*'), "quoted"=>" # not a 'comment'.", "single"=>"\"Howdy!\" he cried.", "hexesc"=>"\r\n is \r\n"}, <<EOY
+    #unicode: "Sosa did fine.\\u263A"
+    #control: "\\b1998\\t1999\\t2000\\n"
+    #hexesc:  "\\x0D\\x0A is \\r\\n"
+    #
+    #single: '"Howdy!" he cried.'
+    #quoted: ' # not a ''comment''.'
+    #tie-fighter: '|\\-*-/|'
+    #EOY
+    #		)
+    #	end
+
 	def test_spec_multiline_scalars
 		# Multiline flow scalars
         map = { 'plain' => 'This unquoted scalar spans many lines.',
 	 		  'quoted' => "So does this quoted scalar.\n" } 
-	 	assert_parse_only( map, <<EOY )
+	 	assert_to_yaml( map, <<EOY )
 plain: This unquoted
        scalar spans
        many lines.
@@ -490,22 +817,34 @@ EOY
 
 	def test_spec_type_int
         map = { 'canonical' => 12345, 'decimal' => 12345, 'octal' => '014'.oct, 'hexadecimal' => '0xC'.hex }
-		assert_parse_only( map, <<EOY )
+		assert_to_yaml( map, <<EOY )
 canonical: 12345
 decimal: +12,345
 octal: 014
 hexadecimal: 0xC
 EOY
         assert_bytecode( map, "D\nM\nScanonical\nS12345\nSdecimal\nS+12,345\nSoctal\nS014\nShexadecimal\nS0xC\nE\n" )
+
+        map = { 'canonical' => 685230, 'decimal' => 685230, 'octal' => '02472256'.oct, 'hexadecimal' => '0x0A74AE'.hex, 'sexagesimal' => 685230 }
+		assert_to_yaml( map, <<EOY )
+canonical: 685230
+decimal: +685,230
+octal: 02472256
+hexadecimal: 0x0A,74,AE
+sexagesimal: 190:20:30
+EOY
+        assert_bytecode( map, "D\nM\nScanonical\nS685230\nSdecimal\nS+685,230\nSoctal\nS02472256\nShexadecimal\nS0x0A,74,AE\nSsexagesimal\nS190:20:30\nE\n" )
+
 	end
 
 	def test_spec_type_float
         map = { 'canonical' => 1230.15, 'exponential' => 1230.15, 'fixed' => 1230.15,
-			  'negative infinity' => -1.0/0.0 }
-		assert_parse_only( map, <<EOY )
+			  'sexagecimal' => 1230.15, 'negative infinity' => -1.0/0.0 }
+		assert_to_yaml( map, <<EOY )
 canonical: 1.23015e+3
 exponential: 12.3015e+02
 fixed: 1,230.15
+sexagecimal: 20:30.15
 negative infinity: -.inf
 EOY
 		nan = YAML::load( <<EOY
@@ -514,12 +853,12 @@ EOY
 		)
 		assert( nan['not a number'].nan? )
 
-        assert_bytecode( map, "D\nM\nScanonical\nS1.23015e+3\nSexponential\nS12.3015e+02\nSfixed\nS1,230.15\nSnegative infinity\nS-.inf\nE\n" )
+        assert_bytecode( map, "D\nM\nScanonical\nS1.23015e+3\nSexponential\nS12.3015e+02\nSfixed\nS1,230.15\nSsexagecimal\nS20:30.15\nSnegative infinity\nS-.inf\nE\n" )
 	end
 
 	def test_spec_type_misc
         map = { nil => nil, true => true, false => false, 'string' => '12345' }
-		assert_parse_only( map, <<EOY )
+		assert_to_yaml( map, <<EOY )
 null: ~
 true: yes
 false: no
@@ -541,7 +880,7 @@ EOY
 				  'description' => 'Super Hoop', 'price' => 2392.00 } ],
 			  'tax' => 251.42, 'total' => 4443.52,
 			  'comments' => "Late afternoon is best. Backup contact is Nancy Billsmer @ 338-4338.\n" }
-		assert_parse_only( invoice, <<EOY )
+		assert_to_yaml( invoice, <<EOY )
 invoice: 34843
 date   : 2001-01-23
 bill-to: &id001
@@ -636,7 +975,8 @@ EOY
 		assert_equals( doc_ct, 3 )
 
         doc_ct = 0
-        yp = YAML::Syck::Parser.new( :Input => :Bytecode )
+        yp = YAML::Parser.new
+        yp.input = :bytecode
         yp.load_documents( 
             "D\nM\nSTime\nS2001-11-23 15:01:42 -05:00\nSUser\nSed\nSWarning\nSThis is an error message for the log file\nN\nE\n" +
             "D\nM\nSTime\nS2001-11-23 15:02:31 -05:00\nSUser\nSed\nSWarning\nSA slightly different error message.\nN\nE\n" +
@@ -681,32 +1021,32 @@ EOY
 		assert_equals( y, { 'invoice' => 34843, 'date' => Date.new( 2001, 1, 23 ), 'total' => 4443.52 } )
 	end
 
-	def test_spec_oneline_docs
-		doc_ct = 0
-		YAML::load_documents( <<EOY
-# The following is a sequence of three documents.
-# The first contains an empty mapping, the second
-# an empty sequence, and the last an empty string.
---- {}
---- [ ]
---- ''
-EOY
-		) { |doc|
-			case doc_ct
-				when 0
-					assert_equals( doc, {} )
-				when 1
-					assert_equals( doc, [] )
-				when 2
-					assert_equals( doc, '' )
-			end
-			doc_ct += 1
-		}
-		assert_equals( doc_ct, 3 )
-	end
+    #	def test_spec_oneline_docs
+    #		doc_ct = 0
+    #		YAML::load_documents( <<EOY
+    ## The following is a sequence of three documents.
+    ## The first contains an empty mapping, the second
+    ## an empty sequence, and the last an empty string.
+    #--- {}
+    #--- [ ]
+    #--- ''
+    #EOY
+    #		) { |doc|
+    #			case doc_ct
+    #				when 0
+    #					assert_equals( doc, {} )
+    #				when 1
+    #					assert_equals( doc, [] )
+    #				when 2
+    #					assert_equals( doc, '' )
+    #			end
+    #			doc_ct += 1
+    #		}
+    #		assert_equals( doc_ct, 3 )
+    #	end
 
 	def test_spec_domain_prefix
-		YAML.add_domain_type( "domain.tld,2002", /(invoice|customer)/ ) { |type, val|
+        customer_proc = proc { |type, val|
 			if Hash === val
                 scheme, domain, type = type.split( ':', 3 )
 				val['type'] = "domain #{type}"
@@ -715,8 +1055,10 @@ EOY
 				raise ArgumentError, "Not a Hash in domain.tld,2002/invoice: " + val.inspect
 			end
 		}
+		YAML.add_domain_type( "domain.tld,2002", 'invoice', &customer_proc ) 
+		YAML.add_domain_type( "domain.tld,2002", 'customer', &customer_proc ) 
         map = { "invoice"=> { "customers"=> [ { "given"=>"Chris", "type"=>"domain customer", "family"=>"Dumars" } ], "type"=>"domain invoice" } }
-		assert_parse_only( map, <<EOY )
+		assert_to_yaml( map, <<EOY )
 # 'http://domain.tld,2002/invoice' is some type family.
 invoice: !domain.tld,2002/^invoice
   # 'seq' is shorthand for 'http://yaml.org/seq'.
@@ -738,7 +1080,7 @@ EOY
 
 	def test_spec_throwaway
         map = {"this"=>"contains three lines of text.\nThe third one starts with a\n# character. This isn't a comment.\n"}
-		assert_parse_only( map, <<EOY )
+		assert_to_yaml( map, <<EOY )
 ### These are four throwaway comment  ###
 
 ### lines (the second line is empty). ###
@@ -759,57 +1101,56 @@ EOY
 	def test_spec_force_implicit
 		# Force implicit
         map = { 'integer' => 12, 'also int' => 12, 'string' => '12' }
-		assert_parse_only( map, <<EOY )
+		assert_to_yaml( map, <<EOY )
 integer: 12
 also int: ! "12"
 string: !str 12
 EOY
-        assert_bytecode( map, "D\nM\nSinteger\nS12\nSalso int\nT!\nS12\nSstring\nT!str\nS12\nE\n" )
 	end
 
-	def test_spec_private_types
-		doc_ct = 0
-		YAML::parse_documents( <<EOY
-# Private types are per-document.
----
-pool: !!ball
-   number: 8
-   color: black
----
-bearing: !!ball
-   material: steel
-EOY
-		) { |doc|
-			case doc_ct
-				when 0
-					assert_equals( doc['pool'].type_id, 'x-private:ball' )
-					assert_equals( doc['pool'].transform.value, { 'number' => 8, 'color' => 'black' } )
-				when 1
-					assert_equals( doc['bearing'].type_id, 'x-private:ball' ) 
-					assert_equals( doc['bearing'].transform.value, { 'material' => 'steel' } )
-			end
-			doc_ct += 1
-		}
-		assert_equals( doc_ct, 2 )
-
-		doc_ct = 0
-		YAML::Syck::Parser.new( :Input => :Bytecode, :Model => :Generic )::load_documents( 
-            "D\nc Private types are per-document.\nM\nSpool\nT!!ball\n" +
-                "M\nSnumber\nS8\nScolor\nSblack\nE\nE\n" +
-            "D\nM\nSbearing\nT!!ball\nM\nSmaterial\nSsteel\nE\nE\n"
-		) { |doc|
-			case doc_ct
-				when 0
-					assert_equals( doc['pool'].type_id, 'x-private:ball' )
-					assert_equals( doc['pool'].transform.value, { 'number' => 8, 'color' => 'black' } )
-				when 1
-					assert_equals( doc['bearing'].type_id, 'x-private:ball' ) 
-					assert_equals( doc['bearing'].transform.value, { 'material' => 'steel' } )
-			end
-			doc_ct += 1
-		}
-		assert_equals( doc_ct, 2 )
-	end
+    #	def test_spec_private_types
+    #		doc_ct = 0
+    #		YAML::parse_documents( <<EOY
+    ## Private types are per-document.
+    #---
+    #pool: !!ball
+    #   number: 8
+    #   color: black
+    #---
+    #bearing: !!ball
+    #   material: steel
+    #EOY
+    #		) { |doc|
+    #			case doc_ct
+    #				when 0
+    #					assert_equals( doc['pool'].type_id, 'x-private:ball' )
+    #					assert_equals( doc['pool'].transform.value, { 'number' => 8, 'color' => 'black' } )
+    #				when 1
+    #					assert_equals( doc['bearing'].type_id, 'x-private:ball' ) 
+    #					assert_equals( doc['bearing'].transform.value, { 'material' => 'steel' } )
+    #			end
+    #			doc_ct += 1
+    #		}
+    #		assert_equals( doc_ct, 2 )
+    #
+    #		doc_ct = 0
+    #		YAML::Syck::Parser.new( :Input => :Bytecode, :Model => :Generic )::load_documents( 
+    #            "D\nc Private types are per-document.\nM\nSpool\nT!!ball\n" +
+    #                "M\nSnumber\nS8\nScolor\nSblack\nE\nE\n" +
+    #            "D\nM\nSbearing\nT!!ball\nM\nSmaterial\nSsteel\nE\nE\n"
+    #		) { |doc|
+    #			case doc_ct
+    #				when 0
+    #					assert_equals( doc['pool'].type_id, 'x-private:ball' )
+    #					assert_equals( doc['pool'].transform.value, { 'number' => 8, 'color' => 'black' } )
+    #				when 1
+    #					assert_equals( doc['bearing'].type_id, 'x-private:ball' ) 
+    #					assert_equals( doc['bearing'].transform.value, { 'material' => 'steel' } )
+    #			end
+    #			doc_ct += 1
+    #		}
+    #		assert_equals( doc_ct, 2 )
+    #	end
 
 	def test_spec_url_escaping
 		YAML.add_domain_type( "domain.tld,2002", "type0" ) { |type, val|
@@ -819,7 +1160,7 @@ EOY
 			"TWO: #{val}"
 		}
         map = { 'same' => [ 'ONE: value', 'ONE: value' ], 'different' => [ 'TWO: value' ] }
-		assert_parse_only( map, <<EOY )
+		assert_to_yaml( map, <<EOY )
 same:
   - !domain.tld,2002/type\\x30 value
   - !domain.tld,2002/type0 value
@@ -834,7 +1175,7 @@ EOY
 		# Override anchor
 		a001 = "The alias node below is a repeated use of this value.\n"
         anc = { 'anchor' => 'This scalar has an anchor.', 'override' => a001, 'alias' => a001 }
-		assert_parse_only( anc, <<EOY )
+		assert_to_yaml( anc, <<EOY )
 anchor : &A001 This scalar has an anchor.
 override : &A001 >
  The alias node below is a
@@ -846,11 +1187,11 @@ EOY
 	end
 
 	def test_spec_explicit_families
-		YAML.add_domain_type( "somewhere.com,2002", /^type$/ ) { |type, val|
+		YAML.add_domain_type( "somewhere.com,2002", 'type' ) { |type, val|
 			"SOMEWHERE: #{val}"
 		}
         map = { 'not-date' => '2002-04-28', 'picture' => "GIF89a\f\000\f\000\204\000\000\377\377\367\365\365\356\351\351\345fff\000\000\000\347\347\347^^^\363\363\355\216\216\216\340\340\340\237\237\237\223\223\223\247\247\247\236\236\236i^\020' \202\n\001\000;", 'hmm' => "SOMEWHERE: family above is short for\nhttp://somewhere.com/type\n" }
-		assert_parse_only( map, <<EOY )
+		assert_to_yaml( map, <<EOY )
 not-date: !str 2002-04-28
 picture: !binary |
  R0lGODlhDAAMAIQAAP//9/X
@@ -891,28 +1232,28 @@ EOY
 		YAML.add_domain_type( "clarkevans.com,2002", 'graph/line', &one_shape_proc )
 		YAML.add_domain_type( "clarkevans.com,2002", 'graph/text', &one_shape_proc )
         seq = [[{"radius"=>7, "center"=>{"x"=>73, "y"=>129}, "TYPE"=>"Shape: graph/circle"}, {"finish"=>{"x"=>89, "y"=>102}, "TYPE"=>"Shape: graph/line", "start"=>{"x"=>73, "y"=>129}}, {"TYPE"=>"Shape: graph/text", "value"=>"Pretty vector drawing.", "start"=>{"x"=>73, "y"=>129}, "color"=>16772795}, "Shape Container"]]
-		assert_parse_only( seq, <<EOY )
+		assert_to_yaml( seq, <<EOY )
 - !clarkevans.com,2002/graph/^shape
   - !^circle
-    center: &ORIGIN {x: 73, y: 129}
+    center: &ORIGIN {'x': 73, 'y': 129}
     radius: 7
   - !^line # !clarkevans.com,2002/graph/line
     start: *ORIGIN
-    finish: { x: 89, y: 102 }
+    finish: { 'x': 89, 'y': 102 }
   - !^text
     start: *ORIGIN
     color: 0xFFEEBB
     value: Pretty vector drawing.
 EOY
         assert_bytecode( seq, "D\nQ\nT!clarkevans.com,2002/graph/^shape\nQ\n" +
-            "T!^circle\nM\nScenter\nAORIGIN\nM\nSx\nS73\nSy\nS129\nE\nSradius\nS7\nE\n" +
-            "T!^line\nc !clarkevans.com,2002/graph/line\nM\nSstart\nRORIGIN\nSfinish\nM\nSx\nS89\nSy\nS102\nE\nE\n" +
+            "T!^circle\nM\nScenter\nAORIGIN\nM\nSx\nS73\nTstr\nSy\nS129\nE\nSradius\nS7\nE\n" +
+            "T!^line\nc !clarkevans.com,2002/graph/line\nM\nSstart\nRORIGIN\nSfinish\nM\nSx\nS89\nTstr\nSy\nS102\nE\nE\n" +
             "T!^text\nM\nSstart\nRORIGIN\nScolor\nS0xFFEEBB\nSvalue\nSPretty vector drawing.\nE\nE\nE\n" )
 	end
 
 	def test_spec_float_explicit
         seq = [ 10.0, 10.0, 10.0, 10.0 ]
-		assert_parse_only( seq, <<EOY )
+		assert_to_yaml( seq, <<EOY )
 # All entries in the sequence
 # have the same type and value.
 - 10.0
@@ -928,7 +1269,7 @@ EOY
 
 	def test_spec_builtin_seq
 		# Assortment of sequences
-		assert_parse_only(
+		assert_to_yaml(
 			{ 'empty' => [], 'in-line' => [ 'one', 'two', 'three', 'four', 'five' ],
 			  'nested' => [ 'First item in top sequence', [ 'Subordinate sequence entry' ],
 			  	"A multi-line sequence entry\n", 'Sixth item in top sequence' ] }, <<EOY
@@ -950,7 +1291,7 @@ EOY
 
 	def test_spec_builtin_map
 		# Assortment of mappings
-		assert_parse_only( 
+		assert_to_yaml( 
 			{ 'empty' => {}, 'in-line' => { 'one' => 1, 'two' => 2 },
 			  'spanning' => { 'one' => 1, 'two' => 2 },
 			  'nested' => { 'first' => 'First entry', 'second' =>
@@ -1012,7 +1353,7 @@ EOY
 
 	def test_spec_builtin_literal_blocks
 		# Assortment of literal scalar blocks
-		assert_parse_only(
+		assert_to_yaml(
 			{"both are equal to"=>"  This has no newline.", "is equal to"=>"The \\ ' \" characters may be\nfreely used. Leading white\n   space is significant.\n\nLine breaks are significant.\nThus this value contains one\nempty line and ends with a\nsingle line break, but does\nnot start with one.\n", "also written as"=>"  This has no newline.", "indented and chomped"=>"  This has no newline.", "empty"=>"", "literal"=>"The \\ ' \" characters may be\nfreely used. Leading white\n   space is significant.\n\nLine breaks are significant.\nThus this value contains one\nempty line and ends with a\nsingle line break, but does\nnot start with one.\n"}, <<EOY
 empty: |
 
@@ -1052,7 +1393,7 @@ EOY
 		str1 = "This has one newline.\n"
 		str2 = "This has no newline."
 		str3 = "This has two newlines.\n\n"
-		assert_parse_only( 
+		assert_to_yaml( 
 			{ 'clipped' => str1, 'same as "clipped" above' => str1, 
 			  'stripped' => str2, 'same as "stripped" above' => str2, 
 			  'kept' => str3, 'same as "kept" above' => str3 }, <<EOY
@@ -1076,7 +1417,7 @@ EOY
 	end
 	
 	def test_spec_span_single_quote
-		assert_parse_only( {"third"=>"a single quote ' must be escaped.", "second"=>"! : \\ etc. can be used freely.", "is same as"=>"this contains six spaces\nand one line break", "empty"=>"", "span"=>"this contains six spaces\nand one line break"}, <<EOY
+		assert_to_yaml( {"third"=>"a single quote ' must be escaped.", "second"=>"! : \\ etc. can be used freely.", "is same as"=>"this contains six spaces\nand one line break", "empty"=>"", "span"=>"this contains six spaces\nand one line break"}, <<EOY
 empty: ''
 second: '! : \\ etc. can be used freely.'
 third: 'a single quote '' must be escaped.'
@@ -1091,7 +1432,7 @@ EOY
 	end
 
 	def test_spec_span_double_quote
-		assert_parse_only( {"is equal to"=>"this contains four  spaces", "third"=>"a \" or a \\ must be escaped.", "second"=>"! : etc. can be used freely.", "empty"=>"", "fourth"=>"this value ends with an LF.\n", "span"=>"this contains four  spaces"}, <<EOY
+		assert_to_yaml( {"is equal to"=>"this contains four  spaces", "third"=>"a \" or a \\ must be escaped.", "second"=>"! : etc. can be used freely.", "empty"=>"", "fourth"=>"this value ends with an LF.\n", "span"=>"this contains four  spaces"}, <<EOY
 empty: ""
 second: "! : etc. can be used freely."
 third: "a \\\" or a \\\\ must be escaped."
@@ -1106,7 +1447,7 @@ EOY
 
 	def test_spec_builtin_time
 		# Time
-		assert_parse_only(
+		assert_to_yaml(
 			{ "space separated" => mktime( 2001, 12, 14, 21, 59, 43, ".10", "-05:00" ), 
 			  "canonical" => mktime( 2001, 12, 15, 2, 59, 43, ".10" ), 
 			  "date (noon UTC)" => Date.new( 2002, 12, 14), 
@@ -1121,7 +1462,7 @@ EOY
 
 	def test_spec_builtin_binary
 		arrow_gif = "GIF89a\f\000\f\000\204\000\000\377\377\367\365\365\356\351\351\345fff\000\000\000\347\347\347^^^\363\363\355\216\216\216\340\340\340\237\237\237\223\223\223\247\247\247\236\236\236iiiccc\243\243\243\204\204\204\377\376\371\377\376\371\377\376\371\377\376\371\377\376\371\377\376\371\377\376\371\377\376\371\377\376\371\377\376\371\377\376\371\377\376\371\377\376\371\377\376\371!\376\016Made with GIMP\000,\000\000\000\000\f\000\f\000\000\005,  \216\2010\236\343@\024\350i\020\304\321\212\010\034\317\200M$z\357\3770\205p\270\2601f\r\e\316\001\303\001\036\020' \202\n\001\000;"
-		assert_parse_only(
+		assert_to_yaml(
 			{ 'canonical' => arrow_gif, 'base64' => arrow_gif, 
 			  'description' => "The binary value above is a tiny arrow encoded as a gif image.\n" }, <<EOY
 canonical: !binary "\\
@@ -1158,17 +1499,6 @@ EOY
 		)
 	end
 
-	def test_perl_regexp
-		# Parsing perl regular expressions from YAML.pm
-		assert_parse_only(
-			[ /bozo$/i ], <<EOY
-- !perl/regexp:
-  REGEXP: bozo$
-  MODIFIERS: i
-EOY
-		)
-	end
-
 	def test_ruby_struct
 		# Ruby structures
 		book_struct = Struct::new( "BookStruct", :author, :title, :year, :isbn )
@@ -1195,6 +1525,12 @@ EOY
     isbn: None
 EOY
 		)
+
+        assert_to_yaml( YAML_Tests::StructTest.new( 123 ), <<EOY )
+--- !ruby/struct:YAML_Tests::StructTest
+c: 123
+EOY
+
 	end
 
 	def test_emitting_indicators
@@ -1235,6 +1571,28 @@ EOY
         )
     end
 
+    #
+    # Test of Ranges
+    #
+    def test_ranges
+
+        # Simple numeric
+        assert_to_yaml( 1..3, <<EOY )
+--- !ruby/range 1..3
+EOY
+
+        # Simple alphabetic
+        assert_to_yaml( 'a'..'z', <<EOY )
+--- !ruby/range a..z
+EOY
+
+        # Float
+        assert_to_yaml( 10.5...30.3, <<EOY )
+--- !ruby/range 10.5...30.3
+EOY
+
+    end
+
     #
     # Tests from Tanaka Akira on [ruby-core]
     #
@@ -1272,12 +1630,45 @@ EOY
     end
 
     #
-    # Circular references
+    # Test Time.now cycle
     #
-    def test_circular_references
-        a = []; a[0] = a; a[1] = a
-        inspect_str = "[[...], [...]]"
-        assert_equals( inspect_str, YAML::load( a.to_yaml ).inspect )
+    def test_time_now_cycle
+        #
+        # From Minero Aoki [ruby-core:2305]
+        #
+        require 'yaml'
+        t = Time.now
+        5.times do
+            assert_equals( t, YAML.load( YAML.dump( t ) ) )
+        end
+    end
+
+    #    #
+    #    # Circular references
+    #    #
+    #    def test_circular_references
+    #        a = []; a[0] = a; a[1] = a
+    #        inspect_str = "[[...], [...]]"
+    #        assert_equals( inspect_str, YAML::load( a.to_yaml ).inspect )
+    #    end
+    
+    #
+    # Test Exception
+    #
+    def test_exception
+        e1 = Exception.new( "hello" )
+        e1.set_backtrace( ["file1.rb", "file2.rb"] )
+        e2 = YAML.load( YAML.dump( e1 ))
+        
+        if RUBY_VERSION.match( /^1\.[012345678]/ )
+            # Ruby 1.8 uses Object.== for Exception.==, so is not valid
+            assert_equals( e1.class, e2.class )
+            assert_equals( e1.backtrace, e2.backtrace )
+            assert_equals( e1.message, e2.message )
+        else
+            # Ruby 1.9 and above
+            assert_equals( e1, e2 )
+        end
     end
 end
 
